from practice_turkish.dictionaries.dictionary import (
    Dictionary,
    DictionaryEntry,
    DictionaryFormatError,
)
from practice_turkish.dictionaries.csvdictionary import CSVDictionaryEntry
from practice_turkish.dictionaries.turkrutdictionary import TurkrutDictionaryEntry
